unit Forza4;

{-----------------------------------------
 TForza4.pas:

 Component to easily implement the Forza4-game in
 your application.

 properties:
   DarkColor:  color of dark pieces
   LightColor:   ''   ' light pieces
   LineColor:  color of lines
   BackColor:  color of BackGround   
   Player1:    Type of player for pl1. (human/cpu)
   Player2:      ''  '   ''    '' pl2.   ''    ''
   GridColumns: No. of columns in grid
   GridRows:    No. of rows in grid
   DrawLines:   Draw the lines yes or no
   WhoIsonturn: Who is next turn?

 events:
   OnfalseMove: Occurs when player clicks on full column
   OnWingame:   Occurs when somebody wins the game

 Important methods:
   NewGame:     Starts a new game.
   Click:       Let a piece drop in the column pointed to
                with the mouse.

Revision history
****************
05-09-97: Started with the component (0.1)
06-09-97: Finished first release of component (1.0)

Information
****************
Made by:   Arjan Scherpenisse, ACS Software
Date:      06-09-97
E-Mail:    A.Scherpenisse@inter.nl.net
Thanks To: Marco Bianchini for the idea of implementing Forza4

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
  mxColumns = 60;
  mxRows    = 60;

type
  TPlayerKind   = (plHuman, plComputer);
  TPlayer       = (ppPlayer1, ppPlayer2, ppNone);

  TBlock = record Player:TPlayer; Filled: Boolean; End;
  TForza4Grid   = Array[0..mxColumns-1,0..mxRows-1] of TBlock;
  TWinGameEvent = procedure(Sender: TObject; Player: TPlayer) of Object;

  TForza4 = class(TGraphicControl)
  private
    fPlaying: Boolean;
    fPlayer1,
    fPlayer2: TPlayerkind;

    fRows,fColumns: Integer;

    fWhosOnTurn:    TPlayer;
    fGrid:          TForza4grid;

    fDarkColor,
    fLightColor,
    fLineColor,
    fBackColor:     TColor;
    fDrawLines:     Boolean;

    fOnWinGame:     TWinGameEvent;
    fOnfalseMove:   TNotifyEvent;

    procedure Paint; override;
    procedure DrawBlock(X,Y: Integer;P:TPlayer);
    procedure ClearGrid;
    procedure SetBlockAt(X,Y:integer; P:TPlayer);
    procedure SetSampleBlocks;
    procedure CheckForWinner;

    procedure fSetRows(N:Integer);
    procedure fSetColumns(N:Integer);
    procedure fSetDarkColor(C:TColor);
    procedure fSetLightColor(C:TColor);
    procedure fSetBackColor(C:TColor);
    procedure fSetLineColor(C:TColor);
    procedure fSetDrawLines(B:Boolean);

  protected
    { Protected declarations }
  public
    procedure Click; override;
    procedure newGame;
    constructor Create(AOwner:TComponent); override;
    { Public declarations }
  published
    property GridRows: Integer read fRows write fSetRows default 9;
    property GridColumns: Integer read fColumns write fSetColumns default 9;
    property WhoIsOnTurn: TPlayer read fWhosOnTurn write fWhosOnTurn;

    property DarkColor: TColor read fDarkColor write fSetDarkColor default clBlue;
    property LightColor: TColor read fLightColor write fSetLightColor default clred;
    property BackColor: TColor read fBackColor write fSetBackColor default clBlack;
    property LineColor: TColor read fLineColor write fSetLineColor default clSilver;

    property DrawLines: Boolean read fDrawLines write fSetDrawLines default True;

    property Player1: TPlayerkind read fPlayer1 write fPlayer1 default plHuman;
    property Player2: TPlayerkind read fPlayer2 write fPlayer2 default plComputer;

    property OnWinGame: TWinGameEvent read fOnWinGame write fOnWinGame;
    property OnFalseMove: TNotifyevent read fOnFalsemove write fOnFalseMove;
  end;

procedure Register;

implementation

// Registratie

procedure Register;
begin
  RegisterComponents('Games', [TForza4]);
end;

// Properties - write-methodes

procedure TForza4.fSetRows(N:Integer);
begin
    If (N>0) and (N<=mxRows-1) then
     begin
       fRows:=N;
       If (csDesigning in Componentstate) then
         SetSampleBlocks;
     end
    else
     MessageDlg('Range [1..60] please...',mtError,[mbRetry],0);
    Paint;
end;

procedure TForza4.fSetColumns(N:Integer);
begin
    If (N>0) and (N<=mxColumns-1) then
     begin
       fColumns:=N;
       If (csDesigning in Componentstate) then
         SetSampleBlocks;
     end
    else
      MessageDlg('Range [1..60] please...',mtError,[mbRetry],0);
    Paint;
end;


procedure TForza4.fSetDarkColor(C:TColor);
begin
    if C<>fLightColor then
      fDarkColor:=C
    else
      MessageDlg('The colors of the pieces cant be the same!',mtError,[mbRetry],0);
    Paint;
end;

procedure TForza4.fSetLightColor(C:TColor);
begin
    If C<>fDarkColor then
      fLightColor:=C
    else
      MessageDlg('The colors of the pieces cant be the same!',mtError,[mbRetry],0);
    Paint;
end;

procedure TForza4.fSetBackColor(C:TColor);
begin
    fbackColor:=C;
    Paint;
end;

procedure TForza4.fSetLineColor(C:TColor);
begin
    fLineColor:=C;
    Paint;
end;


procedure TForza4.fSetDrawLines(B:Boolean);
begin
    fDrawlines:=B;
    Paint;
end;

//Constructor
constructor TForza4.Create(AOwner:TComponent);
Begin
    Inherited create(AOwner);
    fPlaying:=False;
    fRows:=9;fColumns:=9;
    fPlayer1:=plHuman;
    fPlayer2:=plComputer;
    fDarkColor:=clBlue;
    fLightColor:=clred;
    fbackColor:=clBlack;
    fLineColor:=clSilver;
    fDrawLines:=true;
    ClearGrid;

    If (csDesigning in Componentstate) then
      SetSampleBlocks;
End;



Procedure TForza4.ClearGrid;
Var X,Y:Integer;
Begin
  For X:=0 to fColumns-1 do
    For Y:=0 to fRows-1 do
      Begin
        fGrid[X,Y].Filled:=False;
        fGrid[X,Y].Player:=ppNone;
      End;
End;

Procedure TForza4.SetBlockAt(X,Y:integer; P:TPlayer);
Begin
    fGrid[X,Y].Player:=P;
    fGrid[X,Y].Filled:=True;
End;

// Paint - methode


procedure TForza4.Paint;
Var Dx,Dy: real;
    X,Y:Integer;
Begin
    Dx:=Width / fColumns;
    Dy:=Height / fRows;

    Canvas.Pen.Color:=fLineColor;
    Canvas.Brush.style:=bsSolid;
    Canvas.Brush.color:=fbackColor;
    Canvas.Rectangle(0,0,Width,Height);

    If fDrawLines then
     begin
      For X:=1 to fColumns-1 do
        With Canvas do
          Begin
            MoveTo(Round(X*Dx),0); LineTo(Round(X*Dx),Height);
          End;
      For Y:=1 to fRows-1 do
        With Canvas do
          Begin
            MoveTo(0,Round(Y*Dy)); LineTo(Width,Round(Y*Dy));
          End;
     End;
    For X:=0 to fColumns-1 do
      For Y:=0 to fRows-1 do
        If fGrid[X,Y].Filled then
          DrawBlock(X,Y,fGrid[X,Y].Player);
End;

procedure TForza4.DrawBlock(X,Y: Integer;P:TPlayer);
Var Dx,Dy:real;
    C:TColor;
Begin
    If P=ppPlayer1 then
      C:=fLightColor
    else
    if P=ppPlayer2 then
      C:=fDarkColor;

    Dx:=Width / fColumns;
    Dy:=Height / fRows;

    With Canvas do
      begin
        Pen.Color:=C;
        Brush.Color:=C;
        Brush.Style:=bsSolid;
        Ellipse(Round(X*Dx)+2,Round(Y*Dy)+2,Round((X+1)*Dx)-1,Round((Y+1)*Dy)-1);
     end;
End;

Procedure TForza4.SetSampleBlocks;
Var A:Integer;
Begin
    ClearGrid;
    For A:=0 to fColumns-1 do
      If A mod 2 =0 then
        SetBlockAt(A,fRows-1,ppPlayer1)
      else
        SetBlockAt(A,fRows-2,ppPlayer2);
End;



procedure TForza4.Click;
Var TX,Ty: Integer;
    P: TPoint;
begin
  inherited click;
  if fPlaying then
    begin
      GetCursorPos(P);
      P:=ScreenToClient(P);
      Tx:=(P.X div Round(Width/fColumns));
      Ty:=0;
      If  fGrid[tX,tY].Filled=false then
        begin
          While (Ty+1<fRows) and (fGrid[Tx,Ty+1].Filled=false) do
            inc(Ty);
          SetBlockAt(tX,Ty,fWhosOnTurn);
          DrawBlock(tX,tY,fWhosOnTurn);
          CheckForWinner;
          if  fWhosOnTurn=ppPlayer1 then
            fWhosOnTurn:=ppPlayer2
          else
            fWhosOnTurn:=ppPlayer1;
         end
       else
         If Assigned(OnFalseMove) then
           fOnfalseMove(self);
      End
    Else
      MessageBeep(MB_OK);

    If Assigned(OnClick) then
      Onclick(self);
End;

procedure TForza4.CheckForWinner;
Var X,Y:Integer;
    Winner:Boolean;
begin
    Winner:=False;
//  Horizontaal
    For Y:=0 to fRows-1 do
      For X:=0 to fColumns-4 do
        If (fGrid[X,Y].Filled) and
           (fGrid[X,Y].Player=fGrid[X+1,Y].Player) and
           (fGrid[X,Y].Player=fGrid[X+2,Y].Player) and
           (fGrid[X,Y].Player=fGrid[X+3,Y].Player) then
           Winner:=True;
//  verticaal
    For Y:=0 to fRows-4 do
      For X:=0 to fColumns-1 do
        If (fGrid[X,Y].Filled) and
           (fGrid[X,Y].Player=fGrid[X,Y+1].Player) and
           (fGrid[X,Y].Player=fGrid[X,Y+2].Player) and
           (fGrid[X,Y].Player=fGrid[X,Y+3].Player) then
           Winner:=True;
//  Diagonaal: \
    For Y:=0 to fRows-4 do
      For X:=0 to fColumns-4 do
        If (fGrid[X,Y].Filled) and
           (fGrid[X,Y].Player=fGrid[X+1,Y+1].Player) and
           (fGrid[X,Y].Player=fGrid[X+2,Y+2].Player) and
           (fGrid[X,Y].Player=fGrid[X+3,Y+3].Player) then
           Winner:=True;
//  Diagonaal: /
    For Y:=3 to fRows-1 do
      For X:=0 to fColumns-4 do
        If (fGrid[X,Y].Filled) and
           (fGrid[X,Y].Player=fGrid[X+1,Y-1].Player) and
           (fGrid[X,Y].Player=fGrid[X+2,Y-2].Player) and
           (fGrid[X,Y].Player=fGrid[X+3,Y-3].Player) then
           Winner:=True;
    If Winner and Assigned(OnWinGame) then
      Begin
        fonWinGame(self, fWhosOnTurn);
        fPlaying:=false;
      End;
end;

procedure TForza4.newGame;
Begin
    If (fPlaying=false) or (MessageDlg('Stop game in progess?',mtConfirmation,[mbYes,mbNo],0)=mrYes) then
      begin
        ClearGrid;
        Paint;
        fPlaying:=true;
      end;  
End;

end.
